function [s,x]=kac(A,b,tolx,maxiter)
% SYNTAX: [s,x]=kac(A,b,tolx,maxiter)
% The following MATLAB implementation of Kaczmarz algorithm shows how
% easy the algorithm is to implement.
% Inputs:
%          A    = Constraint matrix.
%          b    = right hand side.
%       tolx    = Terminate when the relative diff between two iterations
%                 is less than tolx.
%    maxiter    = Stop after maxiter iterations.
%
% Outputs:
%        x = Solution.
%
% cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

% First, find the size of the matrix.
[m,n]=size(A);
%
% Setup an initial solution of all zeros.
x=zeros(n,1);
iter=0;
%
% Precompute the row norms squared.
n2=zeros(m,1);
for i=1:m,
    n2(i)=norm(A(i,:)',2)^2;
end;
%
% The main loop performs iterations of Kaczmarz algorithm until
% maxiters is exceeded or successive iterates differ by less
% than tolx.
while (1==1);
%
% Check to see whether weve exceeded the maxiters limit.
    iter=iter+1;
    if (iter > maxiter);
        disp('Maximum Iteration Exceeded')
    return;
    end;
%
% Start the update cycle with the current solution.
newx=x;
%
% Perform a cycle of m updates.
    for i=1:m,
        newx=newx-((newx'*A(i,:)'-b(i))/(n2(i)))*A(i,:)';
    end;
%
% Check for convergence to fixed solution.
    if (norm(newx-x)/(1+norm(x)) < tolx),
        return;
    end;
%
% Update x for the next major iteration.
    x=newx;
    s=post2grid(x);
end;
% cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
function g=post2grid(p);
g=zeros(4,4);
% Post to Grid
for i=1:4;
   for j=1:4;
       ndx=4*(i-1)+j;
       g(i,j)=p(ndx);
   end;
end;
% cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
